import turbogears, optparse, sys, urlparse, pkg_resources, os
from turbogears.util import get_project_config
from turbosetup.controllers import SetupController


class SetupCommand:
    """Startup interface to the TurboSetup web-based application configuration system."""    
    desc = "Web-based application configuration"
    need_project = True

    def __init__(self, version):
        parser = optparse.OptionParser(
            usage = "%prog setup [options]", version="%prog " + version)
        parser.add_option("-n", "--no-open",
                          help="don't open the browser automatically",
                          dest="noopen", action="store_true",
                          default = False)
        parser.add_option("-p", "--port",
                          help="port to run the setup site on",
                          dest="port",
                          default=0)
        parser.add_option("-d", "--domain",
                          help="domain name to use when referring to the site",
                          dest="domain",
                          default=None)
        parser.add_option("-u", "--user",
                          help="user name used to log in to the setup site",
                          dest="user",
                          default=None)
        parser.add_option("-w", "--pass",
                          help="password used to log in to the setup site",
                          dest="passw",
                          default=None)
        (options, args) = parser.parse_args(sys.argv[1:])
        self.__dict__.update(options.__dict__)
	self.port = int(options.port)
        self.config = os.path.join(pkg_resources.get_distribution('TurboSetup').location,
                                   'turbosetup',
                                   'setupserver.cfg')

    def run(self):
        """Start the setup server"""
        import cherrypy
	
	package_name = turbogears.util.get_package_name()
	package = __import__(package_name, {}, {}, ["site_setup"])
	try:
	    if not hasattr(package, "site_setup"):
		raise EnvironmentError('Unable to locate a setup module for this application.')
	    if not hasattr(package.site_setup, "SetupRoot"):
		raise EnvironmentError('Unable to locate a setup website for this application.')
	    else:
		root = package.site_setup.SetupRoot
	except EnvironmentError, e:
	    sys.exit(e)

        while not self.user:
            self.user = raw_input("Enter a user name: ")

        while not self.passw:
            self.passw = raw_input("Enter a password: ")

        while self.port == 0:
            port = raw_input("Enter the port this server should run on: ")
            try:
                port = int(port)
                if port > 0 and port < 65355:
                    self.port = port
                else:
                    self.port = 0
            except ValueError:
                self.port = 0

        while not self.domain:
            host = raw_input("Enter the domain name this server is running on [localhost]: ")
            if host == '':
                self.domain = "localhost"
            else:
                self.domain = urlparse.urlparse(host)[1]
                # handle urls entered without the scheme prefix (http://)
                if self.domain == '':
                    self.domain = urlparse.urlparse(host)[2]
                if self.domain == '':
                    self.domain = None

        conf = turbogears.config.config_obj( configfile = self.config )

	cherrypy.tree.mount(root(), "/")

	turbogears.config.update({"global" : {"server.socket_port" : self.port,
					      "server.environment" : 'production',
					      "server.package" : 'setupserver',
					      "setup.uname" : self.user,
					      "setup.passw" : self.passw,
					      "setup.logged_in" : False}})
	if not self.noopen:
	    cherrypy.server.start_with_callback(self.openbrowser)
	else:
	    cherrypy.server.start()
						    
    def openbrowser(self):
        import webbrowser
        webbrowser.open("http://%s:%d/" % (self.domain, self.port))
