from turbogears import widgets

class configDataGrid(widgets.DataGrid):
    """
    Automagic displaying of an entire config on one page.
    """
    def __init__(self, configobj, **kw):
	"""
	configobj is a config object as returned by turbogears.config.config_obj
	or a dictionary of config settings
	"""
	if not type(configobj) is dict:
	    sections = configobj.dict()
	else:
	    sections = configobj
	fields = [(' ', lambda row: row[0]),
		  (' ', lambda row: row[1])]
	defaults = []

	for (key, value) in sections.items():
	    if type(value) is dict:
		defaults.append((key,configDataGrid(value).display()))
	    else:
		defaults.append((key, value))
	super(configDataGrid,self).__init__(name='boo',fields=fields, default=defaults)

class configTableForm(widgets.TableForm):
    """
    Table-based form to edit an entire configuration file
    """
    def __init__(self, configobj, **kw):
	"""
	configobj is a config object as returned by turbogears.config.config_obj
	or a dictionary of config settings
	"""
	
	fields = []
	if not type(configobj) is dict:
	    sections = configobj.dict()
	else:
	    sections = configobj
	for (key, section) in sections.items():
	    fields.append(configSectionFieldSet(section, legend=key))
	super(configTableForm,self).__init__(fields=fields,**kw)


class configListForm(widgets.ListForm):
    """
    List-based form to edit a section of a configuration file
    """
    def __init__(self, configobj, **kw):
	"""
	configobj is a config object as returned by turbogears.config.config_obj
	or a dictionary of config settings
	"""
	fields = []
	if not type(configobj) is dict:
	    sections = configobj.dict()
	else:
	    sections = configobj
	for (key, section) in sections.items():
	    fields.append(configSectionFieldSet(section, legend=key))
	super(configListForm,self).__init__(fields=fields,**kw)


class configSectionFieldSet(widgets.FieldSet):
    """
    FieldSet partial form to edit a section of a configuration file
    """
    def __init__(self, configSection, legend=None, **kw):
	fields = []
	
	for (key, value) in configSection.items():
	    label = key
	    # switch periods for plus signs in form field names (used for field id)
	    key = key.replace('.','+')
	    if type(value) is dict:
		fields.append(configSectionFieldSet(value, legend=key))
	    elif type(value) is int:
		fields.append(widgets.TextField(name=key, label=label, default=str(value)))
	    elif type(value) is bool:
		if value:
		    default = 'on'
		else:
		    default = 'off'
		fields.append(widgets.RadioButtonList(name=key,
						      label=label,
						      default=default,
						      options=[('on', 'on', dict()),
							       ('off', 'off', dict())]))
	    elif type(value) is str:
		fields.append(widgets.TextField(name=key,
						label=label,
						default=value,
						attrs=dict(size=str(len(value)))))
	    elif type(value) is list:
		fields.append(widgets.TextField(name=key,
						label=label,
						default=value,
						attrs=dict(size=(str(len(str(value)))))))
						
	super(configSectionFieldSet,self).__init__(fields=fields, legend=legend, **kw)

class dburiDataGrid(widgets.DataGrid):
    """
    Display the contents of a dburi_builder class ... most likely
    for a confirmation dialog
    """
    def __init__(self, builder, **kw):
	"""
	configobj is a config object as returned by turbogears.config.config_obj
	or a dictionary of config settings
	"""
	if not type(builder) is dict:
	    sections = builder.__dict__
	else:
	    sections = configobj
	fields = [(' ', lambda row: row[0]),
		  (' ', lambda row: row[1])]
	defaults = []

	for (key, value) in sections.items():
	    if type(value) is dict:
		defaults.append((key,configDataGrid(value).display()))
	    else:
		if value is not None:
		    defaults.append((key, value))
		else:
		    defaults.append((key, 'Not Specified'))
	super(dburiDataGrid,self).__init__(name='boo',fields=fields, default=defaults)


class dburiFieldSet(widgets.FieldSet):
    """
    Fieldset built from a dburi_handler instance
    """
    def __init__(self, uriHandler, legend='Database config', **kw):
	fields = []

	for (key, value) in uriHandler.__dict__:
	    if type(value) is int:
		fields.append(widgets.TextField(name=key, label=key, default=str(value)))
	    if type(value) is bool:
		if value:
		    default = 'on'
		else:
		    default = 'off'
		fields.append(widgets.RadioButtonList(name=key,
						      label=key,
						      default=default,
						      options=[('on', 'on', dict()),
							       ('off', 'off', dict())]))
	    if type(value) is str:
		fields.append(widgets.TextField(name=key,
						label=key,
						default=value,
						attrs=dict(size=str(len(value)))))
	super(dburiFieldSet,self).__init__(fields=fields, legend=legend, **kw)
			      
						      
